/*
 * Receive a Mellanox OEM response from BlueField over NC-SI
 */

#include <arpa/inet.h>
#include <linux/if_packet.h>
#include <linux/ip.h>
#include <linux/udp.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <net/if.h>
#include <netinet/ether.h>
#include <unistd.h>

#define DEST_MAC0	0xff
#define DEST_MAC1	0xff
#define DEST_MAC2	0xff
#define DEST_MAC3	0xff
#define DEST_MAC4	0xff
#define DEST_MAC5	0xff

#define ETHER_TYPE	0x88f8

#define DEFAULT_IF	"eth0"
#define BUF_SIZ		1024

void usage()
{
	printf("recv_oem_rsp does not take any arguments.\n");
	printf("Run this program in the background and then send an OEM \
command using send_oem_cmd.\n");
	exit(0);
}

int main(int argc, char *argv[])
{
	char sender[INET6_ADDRSTRLEN];
	int sockfd, ret, i;
	int sockopt;
	ssize_t numbytes;
	struct ifreq ifopts;	/* set promiscuous mode */
	struct ifreq if_ip;	/* get ip addr */
	struct sockaddr_storage ext_addr;
	uint8_t buf[BUF_SIZ];
	char ifName[IFNAMSIZ];

	if (argc > 1)
		usage();

	/* Set the interface name */
	strcpy(ifName, DEFAULT_IF);

	/* Header structures */
	struct ether_header *eh = (struct ether_header *) buf;
	struct iphdr *iph = (struct iphdr *) (buf + sizeof(struct ether_header));
	struct udphdr *udph = (struct udphdr *) (buf + sizeof(struct iphdr) + sizeof(struct ether_header));

	memset(&if_ip, 0, sizeof(struct ifreq));

	/* Open PF_PACKET socket, listening for EtherType ETHER_TYPE */
	if ((sockfd = socket(PF_PACKET, SOCK_RAW, htons(ETHER_TYPE))) == -1) {
		perror("listener: socket");
		return -1;
	}

	/* Set interface to promiscuous mode - do we need to do this every time? */
	strncpy(ifopts.ifr_name, ifName, IFNAMSIZ);
	ioctl(sockfd, SIOCGIFFLAGS, &ifopts);
	ifopts.ifr_flags |= IFF_PROMISC;
	ioctl(sockfd, SIOCSIFFLAGS, &ifopts);
	/* Allow the socket to be reused - incase connection is closed prematurely */
	if (setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, &sockopt, sizeof sockopt) == -1) {
		perror("setsockopt");
		close(sockfd);
		exit(EXIT_FAILURE);
	}
	/* Bind to device */
	if (setsockopt(sockfd, SOL_SOCKET, SO_BINDTODEVICE, ifName, IFNAMSIZ) == -1)	{
		perror("SO_BINDTODEVICE");
		close(sockfd);
		exit(EXIT_FAILURE);
	}

repeat:	numbytes = recvfrom(sockfd, buf, BUF_SIZ, 0, NULL, NULL);

	/* Check the packet is for me */
	if (eh->ether_dhost[0] != DEST_MAC0 ||
			eh->ether_dhost[1] != DEST_MAC1 ||
			eh->ether_dhost[2] != DEST_MAC2 ||
			eh->ether_dhost[3] != DEST_MAC3 ||
			eh->ether_dhost[4] != DEST_MAC4 ||
			eh->ether_dhost[5] != DEST_MAC5) {
		ret = -1;
		goto repeat;
	}

	/* Get source IP */
	((struct sockaddr_in *)&ext_addr)->sin_addr.s_addr = iph->saddr;
	inet_ntop(AF_INET, &((struct sockaddr_in*)&ext_addr)->sin_addr, sender, sizeof sender);

	/* Look up my device IP addr if possible */
	strncpy(if_ip.ifr_name, ifName, IFNAMSIZ);
	if (ioctl(sockfd, SIOCGIFADDR, &if_ip) >= 0) { /* if we can't check then don't */
		/* ignore if I sent it */
		if (strcmp(sender, inet_ntoa(((struct sockaddr_in *)&if_ip.ifr_addr)->sin_addr)) == 0)	{
			ret = -1;
			goto repeat;
		}
	}

	/* UDP payload length */
	ret = ntohs(udph->len) - sizeof(struct udphdr);

	/* Check if this is an OEM response by looking for the MFID */
	for (i = 0; i < numbytes; i++)
		if (buf[i] == 0x81 && buf[i+1] == 0x19)
			break;
	if (i == numbytes)
		goto repeat;

	/* Print packet */
	printf("OEM response packet received:\n");
	for (i = 0; i < numbytes; i++) {
		printf("%02x ", buf[i]);
		if (((i + 1) % 16 == 0) || (i == numbytes - 1))
			printf("\n");
	}

	close(sockfd);
	return ret;
}
